#import "ITKJSVal.h"

@protocol ITKJSPropertyEnumerator
- (BOOL) getNextJSProperty: (JSVal *) slotp;
@end

@interface NSEnumerator (ITKJSPropertyEnumerator) <ITKJSPropertyEnumerator>
@end

@interface ITKJSCountPropertyEnumerator : NSObject <ITKJSPropertyEnumerator>
{
    jsint _count, _i;
}

- initWithCount: (unsigned) count;

@end

/*
   ITKJSProperties is an informal protocol for getting, setting, deleting,
   and enumerating JavaScript properties of Objective-C objects. A class can
   implement all the methods or just specific ones. (NSObject implements none.)

   The default behaviour for Objective-C objects (without implementing anything)
   is as follows:

   Slot "#ivars" is hard-coded to be the C struct containing all the object's
   instance variables. The ITKJSProperties methods will never get called for
   this slot, so you can't redefine its behaviour, sorry.

   Slot 0 is the same as "#ivars" by default (so you can say *o, and o->ivar).
   You can redefine this if you want.

   Slot "name" where name equals the name of an instance variable accesses that
   instance variable. If you assign a function to this slot, it will be taught
   as a method. Other values will be coerced to the instance variable's type.
   Instance variables can't be deleted.

   Slot "name" where name equals the name of a method accesses that method. You
   can call it like a function, or assign a function to it to override its
   implementation. If name refers to both a method and an instance variable,
   read accesses to the slot will generate an object that lazily converts into
   either a method or the value of the instance variable, depending on context.
   Deleting a method causes its former implementation to take effect again.

   Any other slot can be assigned, read and deleted normally, just like for a
   plain JavaScript object.

   Enumerating an object's properties will enumerate over its instance variables
   and also all properties that were explicitly assigned to from JavaScript.

*/

@interface NSObject (ITKJSProperties)

/*
   On entry, vp will contain the current value of the property as remembered by
   JavaScript. You can change it, or leave it alone. Return YES if your method
   already placed the authoritative value of slot into vp, or NO if you want to
   delegate to the default behaviour detailed above.
 */
- (BOOL) getJSProperty: (JSVal) slot value: (JSVal *) vp;

/*
   On entry, vp will contain the assigned value. You can modify it in place
   if it needs to be coerced somehow. Return YES if your method already took all
   necessary actions for assigning to slot, or NO if you still want the default
   behaviour to happen.
 */
- (BOOL) setJSProperty: (JSVal) slot value: (JSVal *) vp;

/*
   On entry, vp will contain JSVAL_TRUE. You can modify it to JSVAL_FALSE if
   slot can't be removed from the object. (To report an error, you can raise
   an exception.) Return YES if your method already took all necessary actions
   for removing slot, or NO if you still want the default behaviour to happen.
 */
- (BOOL) delJSProperty: (JSVal) slot value: (JSVal *) vp;

/*
   If implemented, defines the behaviour of 'for (i in object) ...' style loops.
   The variable i will take on all values returned by the property enumerator's
   getNextJSProperty: method. getNextJSProperty: should return NO when there are
   no more properties to enumerate.
 */
- (id <ITKJSPropertyEnumerator>) jsPropertyEnumerator;

/*
   Optional, needed only for supporting the stupid JS_Enumerate() API which
   returns a pre-enumerated array of slot ids and which Joy doesn't use
   (neither should you).
 */
- (unsigned) jsPropertyCount;

@end
