//--------------------------------------------------------------------------
//  SimpleBrowser.nib/Interpreter.js
//  Copyright (c) 1999, AAA+ Software (info@aaa-plus.com)
//  All rights reserved.
//  Authors: W. Staringer, R. Staringer
//
//  You may freely copy, distribute and reuse the code in this example.
//  AAA+ disclaims any warranty of any kind, expressed or implied,
//  as to its fitness for any particular use.
//--------------------------------------------------------------------------

/*
  This is a simplified version of the class browser contained in every Joy
  command window. It has been updated to use the new language features of Joy 2.
  The source code for the real thing is in
 /LocalLibrary/Frameworks/itkRuntime.framework/Resources/itkBrowser.nib/Interpreter.tcl.
*/

/*
  The example shows how you can use Joy's ability to freely combine
  language features from Objective-C, Java, and JavaScript to your
  advantage.

  The global variable contents corresponds to the current content of
  the class browser. It is a JavaScript (dynamic) array with as many
  elements as there are presently filled columns in the browser.
  Each element is itself a (dynamic) array which contains the class
  objects whose names are displayed in the corresponding column of the
  browser.

  The macro SORTED_DISPLAY controls whether or not code for sorting the
  columns is included in the method definition for loading new columns.

  As is usually done in Joy programming we use the Interpreter object
  (whose scripted implementation is contained in this very file) as a
  generic Controller object for the nib and establish it as the browsers
  delegate. Implementing the two delegate methods

	browser:willDisplayCell:atRow:column:
	browser:numberOfRowsInColumn:

  is very straightforward and all that is required.

*/


#define SORTED_DISPLAY 0 // change to 1 for a sorted display

// initialize the browsers contents to empty
contents = []

/*
  Teach the two skills needed for a passive browser delegate
  to the Controller object (which is the generic JavaScript
  Interpreter object, i.e. us)
*/

@teach self

- (void) browser: sender willDisplayCell: cell
           atRow: (int) row column: (int) column
/*
   skill that gets called each time a new cell is displayed
 */
{
    // get the class object to this display in this column and row
    var clazz = contents[column][row]

    // display the class name in the browser cell
    [cell setStringValue: NSStringFromClass(clazz)]

    // if the class has no subclasses it is a leaf of the browser
    [cell setLeaf: ObjC.subclassesOf(clazz).length == 0]
}


- (int) browser: sender numberOfRowsInColumn: (int) column
/*
   skill that gets called each time a new column is displayed
 */
{
    var superclass = nil, row;

    if (column > 0) {
        // find out which class is selected in the column to
        // the left
	row = [[sender matrixInColumn: column - 1] selectedRow];
	superclass = contents[column - 1][row];
    }
    // assign (sorted or unsorted) vector of subclasses to the
    // column in the contents array
#if SORTED_DISPLAY
    contents[column] = ObjC.subclassesOf(superclass).sort()
#else
    contents[column] = ObjC.subclassesOf(superclass)
#endif      
    return contents[column].length
}

@end


/*
  Initialization
 */

// set the generic Controller (i.e. us, the Javascript Interpreter) as the
// browser's delegate
[browser setDelegate: self]

// for window resizing:
// if the window width gets to small, reduce the number of displayed columns
[browser setMinColumnWidth: 120]

// if the window width gets larger, display more columns to a max of 5
[browser setMaxVisibleColumns: 5]
